unit uConfirmDialog;
{$INCLUDE 'NamedFoldersLite.inc'}
{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  Windows, SysUtils, Classes,
  {$IFDEF FAR3} Plugin3, {$ELSE} PluginW, {$ENDIF} FarKeysW, FarColor,
  uLog, uFAR, uMessages, uConsts, uInfo;

type
  TConfirmOverwriteDialog = class(TFarDialog)
  public
    type TConfirmOverwrite = (coOverwrite, coOverwriteAll, coSkip, coSkipAll, coCancel);
  private
    fContentWidth: integer;
    fMessage: string;
    fItem1Description: string;
    fItem2Description: string;
  private
    fCheckboxRemember: integer;
    fButtonOverwrite: integer;
    fButtonSkip: integer;
  protected
    property ContentWidth: integer read fContentWidth write fContentWidth;
  protected
    class function DefaultContentWidth: integer;
    class function GetDescription(const Description: string; TagMessage: TMessages): string;
  public
    class function ShowModal(const AMessage, AItem1Description, AItem2Description: string): TConfirmOverwrite;
    constructor Create;
    destructor Destroy; override;
    procedure ClearDialog; override;
    function Execute: TConfirmOverwrite;
    property Message: string read fMessage write fMessage;
    property Item1Description: string read fItem1Description write fItem1Description;
    property Item2Description: string read fItem2Description write fItem2Description;
  end;

  TConfirmOverwriteNamedFolderDialog = class(TConfirmOverwriteDialog)
  private
  protected
    class function GetInfoDescription(Info: TNamedFolderInfo; TagMessage: TMessages; NameLength: integer): string;
  public
    class function ShowModal(OldInfo, NewInfo: TNamedFolderInfo): TConfirmOverwriteDialog.TConfirmOverwrite;
  end;

  TConfirmOverwriteFileDialog = class(TConfirmOverwriteDialog)
  private
  protected
    class function GetFileDescription(const FileName: string; TagMessage: TMessages): string;
  public
    class function ShowModal(const FileName: string): TConfirmOverwriteDialog.TConfirmOverwrite;
  end;

implementation

{ TConfirmOverwriteDialog }

procedure TConfirmOverwriteDialog.ClearDialog;
begin
  inherited;
  fCheckboxRemember := -1;
  fButtonOverwrite := -1;
  fButtonSkip := -1;
end;

constructor TConfirmOverwriteDialog.Create;
begin
  inherited Create( {$IFDEF FAR3_UP} PLUGIN_GUID, DIALOG_CONFIRMOVERWRITE_GUID {$ELSE} FarApi.ModuleNumber {$ENDIF} );
  fContentWidth := DefaultContentWidth;
  //Log('ContentWidth = %d', [fContentWidth]);
end;

class function TConfirmOverwriteDialog.DefaultContentWidth: integer;
begin
  Result := TFarUtils.GetConsoleWidth - 14;
end;

destructor TConfirmOverwriteDialog.Destroy;
begin
  inherited;
end;

function TConfirmOverwriteDialog.Execute: TConfirmOverwrite;
var
  FinalMessage, FinalItem1Description, FinalItem2Description: string;
  Button, Height: integer;
begin
  Result := coCancel;
  FinalMessage := Copy(Message, 1, ContentWidth);
  FinalItem1Description := Copy(Item1Description, 1, ContentWidth);
  FinalItem2Description := Copy(Item2Description, 1, ContentWidth);
  // Build the dialog
                       AddLabel    (  5,  2,       PFarChar(FinalMessage), 0);
  if (Item1Description <> '') or (Item2Description <> '') then
                       AddLabel    (  0,  LastY+1, nil, DIF_SEPARATOR);
  if Item1Description <> '' then
                       AddLabel    (  5,  LastY+1, PFarChar(FinalItem1Description), 0);
  if Item2Description <> '' then
                       AddLabel    (  5,  LastY+1, PFarChar(FinalItem2Description), 0);
                       AddLabel    (  0,  LastY+1, nil, DIF_SEPARATOR);
  fCheckboxRemember := AddCheckbox (  5,  LastY+1, GetMsg(MConfirmOverwriteRemember), False, 0);
                       AddLabel    (  0,  LastY+1, nil, DIF_SEPARATOR);
  fButtonOverwrite :=  AddButton   (  0,  LastY+1, GetMsg(MConfirmOverwriteOverwrite), DIF_CENTERGROUP {$IFDEF FAR3_UP} or DIF_FOCUS {$ENDIF} );
  fButtonSkip :=       AddButton   (  0,  LastY,   GetMsg(MConfirmOverwriteSkip), DIF_CENTERGROUP);
                       AddButton   (  0,  LastY,   GetMsg(MCancel), DIF_CENTERGROUP);
  Height := LastY;
                       AddDoubleBox(  3,  1, ContentWidth + 4, Height + 1, GetMsg(MConfirmOverwriteTitle), 0);
  // Set focus
  {$IFNDEF FAR3_UP}
  Items[fButtonOverwrite].Focus := 1;
  {$ENDIF}
  // Set the default button
  {$IFDEF FAR3_UP}
  Items[fButtonOverwrite].Flags := Items[fButtonOverwrite].Flags or DIF_DEFAULTBUTTON;
  {$ELSE}
  Items[fButtonOverwrite].DefaultButton := 1;
  {$ENDIF}
  // Show the dialog
  if Build(-1, -1, ContentWidth + 8, Height + 3, FDLG_WARNING, HELP_EDITOR) then begin
    if inherited Execute(Button) then begin
      if Button = fButtonOverwrite then begin
        if ItemChecked[fCheckboxRemember] then
          Result := coOverwriteAll
        else
          Result := coOverwrite;
      end
      else if Button = fButtonSkip then begin
        if ItemChecked[fCheckboxRemember] then
          Result := coSkipAll
        else
          Result := coSkip;
      end;
    end;
  end;
end;

class function TConfirmOverwriteDialog.GetDescription(const Description: string; TagMessage: TMessages): string;
begin
  Result := Format('%s %s', [GetMsgPas(TagMessage), Description]);
end;

class function TConfirmOverwriteDialog.ShowModal(const AMessage, AItem1Description, AItem2Description: string): TConfirmOverwrite;
var
  Dialog: TConfirmOverwriteDialog;
begin
  Dialog := TConfirmOverwriteDialog.Create;
  try
    Dialog.Message := AMessage;
    Dialog.Item1Description := AItem1Description;
    Dialog.Item2Description := AItem2Description;
    Result := Dialog.Execute;
  finally
    FreeAndNil(Dialog);
  end;
end;

{ TConfirmOverwriteNamedFolderDialog }

class function TConfirmOverwriteNamedFolderDialog.GetInfoDescription(Info: TNamedFolderInfo; TagMessage: TMessages; NameLength: integer): string;
begin
  Result := GetDescription(Format('%*.*s   %s', [NameLength, NameLength, Info.Name, Info.DisplayableDescription]), TagMessage);
end;

class function TConfirmOverwriteNamedFolderDialog.ShowModal(OldInfo, NewInfo: TNamedFolderInfo): TConfirmOverwriteDialog.TConfirmOverwrite;
var
  NameLength, NewNameLength: integer;
begin
  NameLength := Length(OldInfo.Name);
  NewNameLength := Length(NewInfo.Name);
  if NameLength < NewNameLength then
    NameLength := NewNameLength;
  Result := inherited ShowModal(
              GetMsgPas(MConfirmOverwriteFile),
              GetInfoDescription(OldInfo, MConfirmOverwriteTagNew, NameLength),
              GetInfoDescription(NewInfo, MConfirmOverwriteTagOld, NameLength)
            );
end;

{ TConfirmOverwriteFileDialog }

class function TConfirmOverwriteFileDialog.GetFileDescription(const FileName: string; TagMessage: TMessages): string;
var
  SpacesNeeded: integer;
  FileInfo, DateTimeFormat: string;
  SR: TSearchRec;
  {$IFNDEF DELPHIXE2_UP}
  SysTime: TSystemTime;
  {$ENDIF}
  ModificationDate: TDateTime;
begin
  Result := GetDescription('', TagMessage);
  if FindFirst(FileName, faAnyFile, SR) = 0 then
    try
      if (SR.Attr and faDirectory) <> 0 then
        FileInfo := GetMsgPas(MDirectorySize)
      else
        FileInfo := FormatCurr('###,###,###,##0', SR.Size);
      DateTimeFormat := GetMsgPas(MDateTimeFormat);
      if DateTimeFormat = '' then
        DateTimeFormat := 'c';
      {$IFDEF DELPHIXE2_UP}
      ModificationDate := SR.Timestamp;
      {$ELSE}
      if FileTimeToSystemTime(SR.FindData.ftLastWriteTime, SysTime) then
        ModificationDate := EncodeDate(SysTime.wYear, SysTime.wMonth, SysTime.wDay) + EncodeTime(SysTime.wHour, SysTime.wMinute, SysTime.wSecond, SysTime.wMilliseconds)
      else
        ModificationDate := 0;
      {$ENDIF}
      FileInfo := FileInfo + ' ' + FormatDateTime(DateTimeFormat, ModificationDate);
      SpacesNeeded := DefaultContentWidth - Length(Result) - Length(FileInfo) - 2;
      if SpacesNeeded > 0 then
        FileInfo := StringOfChar(' ', SpacesNeeded) + FileInfo;
    finally
      FindClose(SR);
    end
  else
    FileInfo := GetMsgPas(MFailedToReadFileInfo);
  Result := Result + FileInfo;
end;

class function TConfirmOverwriteFileDialog.ShowModal(const FileName: string): TConfirmOverwriteDialog.TConfirmOverwrite;
begin
  Result := inherited ShowModal(
              GetMsgPas(MConfirmOverwriteFile),
              '',
              GetFileDescription(FileName, MConfirmOverwriteTagOld)
            );
end;

end.
