unit uNamedFolders;
{$INCLUDE 'NamedFoldersLite.inc'}

interface

uses
  Windows, SysUtils, Classes,
  {$IFDEF FAR3} Plugin3, {$ELSE} PluginW, {$ENDIF} FarKeysW, FarColor,
  uLog, uFAR;

procedure SetStartupInfoW(const Info: TPluginStartupInfo); stdcall;
procedure ExitFARW( {$IFDEF FAR3_UP} const Info: TExitInfo {$ENDIF} ); stdcall;
{$IFDEF FAR3_UP}
procedure GetGlobalInfoW(var Info: TGlobalInfo); stdcall;
function OpenW(var Info: TOpenInfo): THandle; stdcall;
procedure GetOpenPanelInfoW(var Info: TOpenPanelInfo); stdcall;
procedure ClosePanelW(const Info: TClosePanelInfo); stdcall;
function GetFindDataW(var Info: TGetFindDataInfo): TIntPtr; stdcall;
procedure FreeFindDataInfo(const Info: TFreeFindDataInfo); stdcall;
function MakeDirectoryW(const Info: TMakeDirectoryInfo): TIntPtr; stdcall;
function SetDirectoryW(const Info: TSetDirectoryInfo): TIntPtr; stdcall;
function DeleteFilesW(const Info: TDeleteFilesInfo): TIntPtr; stdcall;
function GetFilesW(var Info: TGetFilesInfo): TIntPtr; stdcall;
function PutFilesW(var Info: TPutFilesInfo): TIntPtr; stdcall;
function ProcessPanelInputW(const Info: TProcessPanelInputInfo): TIntPtr; stdcall;
function ProcessPanelEventW(const Info: TProcessPanelEventInfo): TIntPtr; stdcall;
{$ELSE}
function OpenPluginW(OpenFrom: integer; Item: integer): THandle; stdcall;
procedure GetOpenPluginInfoW(Plugin: THandle; var Info: TOpenPluginInfo); stdcall;
procedure ClosePluginW(Info: THandle); stdcall;
function GetFindDataW(Plugin: THandle; var PanelItem: PPluginPanelItemArray; var ItemsNumber: integer; OpMode: integer): TIntPtr; stdcall;
procedure FreeFindDataInfo(Plugin: THandle; PanelItem: PPluginPanelItemArray; var ItemsNumber: integer); stdcall;
function MakeDirectoryW(Plugin: THandle; var Name: PFarChar; OpMode: integer): TIntPtr; stdcall;
function SetDirectoryW(Plugin: THandle; Dir: PFarChar; OpMode: integer): TIntPtr; stdcall;
function DeleteFilesW(Plugin: THandle; PanelItem: PPluginPanelItem; ItemsNumber: integer; OpMode: integer): TIntPtr; stdcall;
function GetFilesW(Plugin: THandle; PanelItem: PPluginPanelItem; ItemsNumber: integer; Move: integer; var DestPath: PFarChar; OpMode: integer): TIntPtr; stdcall;
function PutFilesW(Plugin: THandle; PanelItem: PPluginPanelItem; ItemsNumber: integer; Move: integer; SrcPath: PFarChar; OpMode: integer): TIntPtr; stdcall;
function ProcessKeyW(Plugin: THandle; Key: integer; ControlState: DWORD): integer; stdcall;
function ProcessEventW(Plugin: THandle; Event: integer; Param: Pointer): TIntPtr; stdcall;
{$ENDIF}
procedure GetPluginInfoW(var Info: TPluginInfo); stdcall;
function ConfigureW( {$IFDEF FAR3_UP} var Info: TConfigureInfo {$ELSE} ItemNumber: integer {$ENDIF} ): TIntPtr; stdcall;

implementation

uses
  uSystem, uConsts, uMessages, uOptions, uOptionsDialog, uMakeDirDialog, uTasks, uPanel;

const
  PLUGIN_TITLE = 'Named Folders Lite';
  PLUGIN_DESCRIPTION = 'Named Folders Lite';
  PLUGIN_AUTHOR = 'Pepak, http://www.pepak.net';

procedure HandleException(E: Exception; const Source: string);
var
  ExceptionMsg1, ExceptionMsg2: string;
begin
  if E is EAbort then
    Exit;
  {$IFDEF DEBUG_LOGS}
  Log('%s: Exception %s: %s', [Source, E.ClassName, E.Message]);
  {$ENDIF}
  ExceptionMsg1 := Format(GetMsgPas(MExceptionOccurred), [E.ClassName]);
  ExceptionMsg2 := E.Message;
  FarMessage([MTitle, MExtraString1, MExtraString2, MExtraString3, MOk], [PFarChar(ExceptionMsg1), PFarChar(ExceptionMsg2), PFarChar(Source)], FMSG_WARNING, 1);
end;

procedure SetStartupInfoW(const Info: TPluginStartupInfo);
begin
  {$IFDEF DEBUG_LOGS}
  Log('SetStartupInfo START');
  {$ENDIF}
  FarApi := Info;
  FSF := Info.FSF^;
  FarApi.FSF := @FSF;
  {$IFDEF DEBUG_LOGS}
  Log('SetStartupInfo END');
  {$ENDIF}
end;

procedure ExitFARW( {$IFDEF FAR3_UP} const Info: TExitInfo {$ENDIF} );
begin
  {$IFDEF DEBUG_LOGS}
  Log('ExitFARW START');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
procedure GetGlobalInfoW(var Info: TGlobalInfo);
var
  FN: string;
  VerHigh, VerLow, VerBuild: LongWord;
begin
  {$IFDEF DEBUG_LOGS}
  Log('GetGlobalInfo START');
  {$ENDIF}
  FillChar(Info, Sizeof(Info), 0);
  Info.StructSize := Sizeof(Info);
  Info.MinFarVersion := MakeFarVersion( {$IFDEF FAR3_UP} 3, 0, 0, 0, 0 {$ELSE} 2, 0, 1807 {$ENDIF} );
  FN := GetModuleName(HInstance);
  if GetFileVersion(FN, VerHigh, VerLow, VerBuild) then
    Info.Version := MakeFarVersion( {$IFDEF FAR3_UP} VerHigh, VerLow, 0, VerBuild, 0 {$ELSE} VerHigh, VerLow, VerBuild {$ENDIF} )
  else
    Info.Version := MakeFarVersion( {$IFDEF FAR3_UP} 0, 0, 0, 0, 0 {$ELSE} 0, 0 ,0 {$ENDIF} );
  Info.Guid := PLUGIN_GUID;
  Info.Title := PLUGIN_TITLE;
  Info.Description := PLUGIN_DESCRIPTION;
  Info.Author := PLUGIN_AUTHOR;
  {$IFDEF DEBUG_LOGS}
  Log('GetGlobalInfo END');
  {$ENDIF}
end;
{$ENDIF}

var
  PluginInfoInitialized: boolean = False;
  PanelMenuStrings:  packed array[0..0] of PFarChar;
  PanelMenuGUIDs:    packed array[0..0] of TGUID;
  ConfigMenuStrings: array[0..0] of PFarChar;
  ConfigMenuGUIDs:   array[0..0] of TGUID;

procedure GetPluginInfoW(var Info: TPluginInfo);
begin
  {$IFDEF DEBUG_LOGS}
  Log('GetPluginInfoW START');
  {$ENDIF}
  if not PluginInfoInitialized then begin
    PanelMenuStrings[0] := GetMsg(MTitle);
    PanelMenuGUIDs[0] := MENU_DISK_GUID;
    ConfigMenuStrings[0] := GetMsg(MTitle);
    ConfigMenuGUIDs[0] := MENU_CONFIG_GUID;
    PluginInfoInitialized := True;
  end;
  FillChar(Info, Sizeof(Info), 0);
  Info.StructSize := Sizeof(Info);
  Info.CommandPrefix := 'cd';
  {$IFDEF FAR3_UP}
  Info.DiskMenu.Count := Length(PanelMenuStrings);
  Info.DiskMenu.Strings := @PanelMenuStrings[0];
  Info.DiskMenu.Guids := @PanelMenuGUIDs[0];
  Info.PluginConfig.Count := Length(ConfigMenuStrings);
  Info.PluginConfig.Strings := @ConfigMenuStrings[0];
  Info.PluginConfig.Guids := @ConfigMenuGUIDs[0];
  {$ELSE}
  Info.DiskMenuStringsNumber := Length(PanelMenuStrings);
  Info.DiskMenuStrings := @PanelMenuStrings[0];
  Info.PluginConfigStringsNumber := Length(ConfigMenuStrings);
  Info.PluginConfigStrings := @ConfigMenuStrings[0];
  {$ENDIF}
  {$IFDEF DEBUG_LOGS}
  Log('GetPluginInfoW END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
function OpenW(var Info: TOpenInfo): THandle;
{$ELSE}
function OpenPluginW(OpenFrom: integer; Item: integer): THandle;
{$ENDIF}
var
  CommandLine: string;
begin
  {$IFDEF DEBUG_LOGS}
  Log('OpenW START');
  {$ENDIF}
  {$IFDEF DEBUG_LOGS}
  Log('OpenFrom = %d', [ {$IFDEF FAR3_UP} Info.OpenFrom {$ELSE} OpenFrom {$ENDIF} ]);
  {$ENDIF}
  Result := 0;
  try
    if {$IFDEF FAR3_UP} Info.OpenFrom = OPEN_COMMANDLINE {$ELSE} OpenFrom = OPEN_COMMANDLINE {$ENDIF} then begin
      CommandLine := TFarUtils.CopyString( {$IFDEF FAR3_UP} POpenCommandLineInfo(Info.Data)^.CommandLine {$ELSE} PFarChar(Item) {$ENDIF} );
      {$IFDEF DEBUG_LOGS}
      Log('Command line = %s', [CommandLine]);
      {$ENDIF}
      if CommandLine = '' then begin
        {$IFDEF DEBUG_LOGS}
        Log('Opening panel from command line');
        {$ENDIF}
        Result := THandle(TNamedFolders.OpenPanel);
        {$IFDEF DEBUG_LOGS}
        Log('Result = 0x%08.8x', [Result]);
        {$ENDIF}
      end
      else begin
        TNamedFolders.ProcessCommandLine(CommandLine);
      end;
    end
    {$IFDEF FAR3_UP}
    else if Info.OpenFrom = OPEN_FROMMACRO_ then begin
      {$IFDEF DEBUG_LOGS}
      Log('Opening from macro');
      {$ENDIF}
      Result := THandle(TNamedFolders.HandleMacroCall(POpenMacroInfo(Info.Data)));
      {$IFDEF DEBUG_LOGS}
      Log('Result = %d', [Result]);
      {$ENDIF}
    end
    {$ENDIF}
    else if {$IFDEF FAR3_UP} Info.OpenFrom = OPEN_SHORTCUT {$ELSE} OpenFrom = OPEN_SHORTCUT {$ENDIF} then begin
      {$IFDEF DEBUG_LOGS}
      Log('Opening from shortcut');
      {$ENDIF}
      Result := THandle(TNamedFolders.OpenPanel);
      {$IFDEF DEBUG_LOGS}
      Log('Result = 0x%08.8x', [Result]);
      {$ENDIF}
    end
    else if {$IFDEF FAR3_UP} Info.OpenFrom = OPEN_DISKMENU {$ELSE} OpenFrom = OPEN_DISKMENU {$ENDIF} then begin
      {$IFDEF DEBUG_LOGS}
      Log('Opening in left panel');
      {$ENDIF}
      Result := THandle(TNamedFolders.OpenPanel);
      {$IFDEF DEBUG_LOGS}
      Log('Result = 0x%08.8x', [Result]);
      {$ENDIF}
    end
    else if {$IFDEF FAR3_UP} Info.OpenFrom = OPEN_RIGHTDISKMENU {$ELSE} False {$ENDIF} then begin
      {$IFDEF DEBUG_LOGS}
      Log('Opening in right panel');
      {$ENDIF}
      Result := THandle(TNamedFolders.OpenPanel);
      {$IFDEF DEBUG_LOGS}
      Log('Result = 0x%08.8x', [Result]);
      {$ENDIF}
    end;
  except
    on E: Exception do
      HandleException(E, {$IFDEF FAR3_UP} 'OpenW' {$ELSE} 'OpenPluginW' {$ENDIF} );
  end;
  {$IFDEF DEBUG_LOGS}
  Log('OpenW END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
procedure ClosePanelW(const Info: TClosePanelInfo);
{$ELSE}
procedure ClosePluginW(Info: THandle);
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
begin
  {$IFDEF DEBUG_LOGS}
  Log('ClosePanelW START');
  {$ENDIF}
  try
    //Log('Closing panel %d', [Info.hPanel]);
    if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Info {$ENDIF} , Panel) then
      Exit;
    {$IFDEF DEBUG_LOGS}
    Log('Panel is Named Folders Panel');
    {$ENDIF}
    FreeAndNil(Panel);
  except
    on E: Exception do
      HandleException(E, {$IFDEF FAR3_UP} 'ClosePanelW' {$ELSE} 'ClosePluginW' {$ENDIF} );
  end;
  {$IFDEF DEBUG_LOGS}
  Log('ClosePanelW END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
procedure GetOpenPanelInfoW(var Info: TOpenPanelInfo);
{$ELSE}
procedure GetOpenPluginInfoW(Plugin: THandle; var Info: TOpenPluginInfo);
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
begin
  {$IFDEF DEBUG_LOGS}
  Log('GetOpenPanelInfoW START');
  {$ENDIF}
  try
    //Log('Loading panel info %d', [Plugin]);
    if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then
      Exit;
    {$IFDEF DEBUG_LOGS}
    Log('Panel is Named Folders Panel');
    {$ENDIF}
    {$IFDEF DEBUG_LOGS}
    Log('Directory = %s', [Panel.FarDirectory]);
    {$ENDIF}
    FillChar(Info, Sizeof(Info), 0);
    Info.StructSize := Sizeof(Info);
    {$IFDEF FAR3_UP}
    Info.hPanel := Panel.Handle;
    {$ENDIF}
    Info.Flags := OPIF_RAWSELECTION
               or OPIF_SHOWNAMESONLY
               or OPIF_SHOWRIGHTALIGNNAMES
               or OPIF_SHOWPRESERVECASE
               {$IFDEF FAR3_UP}
               or OPIF_DISABLESORTGROUPS
               //or OPIF_DISABLEHIGHLIGHTING
               {$ENDIF}
               ;
    Info.CurDir := Panel.FarDirectory;
    Info.Format := GetMsg(MTitle);
    Info.PanelTitle := Panel.FarTitle;
    Info.StartPanelMode := Ord('6');
  except
    on E: Exception do
      HandleException(E, {$IFDEF FAR3_UP} 'GetOpenPanelInfoW' {$ELSE} 'GetOpenPluginInfoW' {$ENDIF} );
  end;
  {$IFDEF DEBUG_LOGS}
  Log('GetOpenPanelInfoW END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
function GetFindDataW(var Info: TGetFindDataInfo): TIntPtr;
{$ELSE}
function GetFindDataW(Plugin: THandle; var PanelItem: PPluginPanelItemArray; var ItemsNumber: integer; OpMode: integer): TIntPtr;
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
  //i: integer;
begin
  {$IFDEF DEBUG_LOGS}
  Log('GetFindDataW START');
  {$ENDIF}
  Result := 0;
  try
    //Log('Loading panel content %d', [Info.hPanel]);
    {$IFDEF FAR3_UP}
    if Info.StructSize >= Sizeof(Info) then begin
    {$ELSE}
    begin
    {$ENDIF}
      if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then
        Exit;
      if ({$IFDEF FAR3_UP} Info. {$ENDIF} OpMode = {$IFDEF FAR3_UP} OPM_NONE {$ELSE} 0 {$ENDIF} ) then begin
        Panel.Clear;
        {$IFDEF FAR3_UP} Info. {$ENDIF} PanelItem := Panel.GetPanelItems;
        {$IFDEF FAR3_UP} Info. {$ENDIF} ItemsNumber := Panel.GetPanelItemsCount;
        //Log(' - Item count = %d', [Info.ItemsNumber]);
        //for i := 0 to Pred(Info.ItemsNumber) do
          //Log(' - Item %d: %s | %s', [i, string(Info.PanelItem^[i].FileName), string(Info.PanelItem^[i].Description)]);
        Result := 1;
      end;
    end;
  except
    on E: Exception do
      HandleException(E, 'GetFindDataW');
  end;
  {$IFDEF DEBUG_LOGS}
  Log('GetFindDataW END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
procedure FreeFindDataInfo(const Info: TFreeFindDataInfo);
{$ELSE}
procedure FreeFindDataInfo(Plugin: THandle; PanelItem: PPluginPanelItemArray; var ItemsNumber: integer);
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
begin
  {$IFDEF DEBUG_LOGS}
  Log('FreeFindDataInfo START');
  {$ENDIF}
  if TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then begin
    if Panel.FileToBeSelected <> '' then begin
      try
        Panel.SelectFile(Panel.FileToBeSelected);
      finally
        Panel.FileToBeSelected := '';
      end;
    end;
  end;
  {$IFDEF DEBUG_LOGS}
  Log('FreeFindDataInfo END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
function MakeDirectoryW(const Info: TMakeDirectoryInfo): TIntPtr;
{$ELSE}
function MakeDirectoryW(Plugin: THandle; var Name: PFarChar; OpMode: integer): TIntPtr;
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
  DirName: string;
begin
  {$IFDEF DEBUG_LOGS}
  Log('MakeDirectoryW START');
  {$ENDIF}
  Result := -1;
  try
    {$IFDEF FAR3_UP}
    if Info.StructSize >= Sizeof(Info) then begin
    {$ELSE}
    begin
    {$ENDIF}
      if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then
        Exit;
      {$IFDEF DEBUG_LOGS}
      Log('Panel is Named Folders Panel');
      {$ENDIF}
      DirName := TFarUtils.CopyString( {$IFDEF FAR3_UP} Info. {$ENDIF} Name);
      {$IFDEF DEBUG_LOGS}
      Log('Directory = %s', [DirName]);
      {$ENDIF}
      if TMakeDirDialog.RunDialog(DirName) then
        if Panel.MakeDirectory(DirName) then
          Result := 1
        else
          Result := 0
      else
        Result := -1;
      {$IFDEF DEBUG_LOGS}
      Log('Result = %d', [Result]);
      {$ENDIF}
    end;
  except
    on E: Exception do
      HandleException(E, 'MakeDirectoryW');
  end;
  {$IFDEF DEBUG_LOGS}
  Log('MakeDirectoryW END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
function SetDirectoryW(const Info: TSetDirectoryInfo): TIntPtr;
{$ELSE}
function SetDirectoryW(Plugin: THandle; Dir: PFarChar; OpMode: integer): TIntPtr;
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
  sDir: string;
begin
  {$IFDEF DEBUG_LOGS}
  Log('SetDirectoryW START');
  {$ENDIF}
  Result := 0;
  try
    {$IFDEF FAR3_UP}
    if Info.StructSize >= Sizeof(Info) then begin
    {$ELSE}
    begin
    {$ENDIF}
      if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then
        Exit;
      {$IFDEF DEBUG_LOGS}
      Log('Panel is Named Folders Panel');
      {$ENDIF}
      sDir := TFarUtils.CopyString( {$IFDEF FAR3_UP} Info. {$ENDIF} Dir);
      {$IFDEF DEBUG_LOGS}
      Log('Directory = %s', [sDir]);
      {$ENDIF}
      if sDir <> '..' then
        sDir := '/' + sDir;
      if Panel.ChangeDirectory(sDir) then
        Result := 1
      else
        Result := 0;
      {$IFDEF DEBUG_LOGS}
      Log('Result = %d', [Result]);
      {$ENDIF}
    end;
  except
    on E: Exception do
      HandleException(E, 'SetDirectoryW');
  end;
  {$IFDEF DEBUG_LOGS}
  Log('SetDirectoryW END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
function DeleteFilesW(const Info: TDeleteFilesInfo): TIntPtr;
{$ELSE}
function DeleteFilesW(Plugin: THandle; PanelItem: PPluginPanelItem; ItemsNumber: integer; OpMode: integer): TIntPtr;
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
  Item: PPluginPanelItem;
  i: Integer;
begin
  {$IFDEF DEBUG_LOGS}
  Log('DeleteFilesW START');
  {$ENDIF}
  Result := 0;
  try
    {$IFDEF FAR3_UP}
    if Info.StructSize >= Sizeof(Info) then begin
    {$ELSE}
    begin
    {$ENDIF}
      if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then
        Exit;
      {$IFDEF DEBUG_LOGS}
      Log('Panel is Named Folders Panel');
      {$ENDIF}
      Item := {$IFDEF FAR3_UP} Info.PanelItem {$ELSE} PanelItem {$ENDIF} ;
      if {$IFDEF FAR3_UP} Info.ItemsNumber {$ELSE} ItemsNumber {$ENDIF} > 0 then begin
        if ( {$IFDEF FAR3_UP} Info.OpMode {$ELSE} OpMode {$ENDIF} and OPM_SILENT) = 0 then
          if FarMessage([MTitle, MConfirmDeletion, MYes, MNo, MCancel], FMSG_WARNING, 3) <> 0 then
            Exit;
        for i := 0 to Pred( {$IFDEF FAR3_UP} Info.ItemsNumber {$ELSE} ItemsNumber {$ENDIF} ) do begin
          if Panel.DeleteNamedFolder(TFarUtils.CopyString(Item^. {$IFDEF FAR3_UP} FileName {$ELSE} FindData.cFileName {$ENDIF} )) then
            Result := 1;
          Inc(Item);
        end;
      end;
    end;
  except
    on E: Exception do
      HandleException(E, 'DeleteFilesW');
  end;
  {$IFDEF DEBUG_LOGS}
  Log('DeleteFilesW START');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
function GetFilesW(var Info: TGetFilesInfo): TIntPtr;
{$ELSE}
function GetFilesW(Plugin: THandle; PanelItem: PPluginPanelItem; ItemsNumber: integer; Move: integer; var DestPath: PFarChar; OpMode: integer): TIntPtr;
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
  Item: PPluginPanelItem;
  OverwriteState: TNamedFoldersPanel.TOverwriteState;
  i: Integer;
begin
  {$IFDEF DEBUG_LOGS}
  Log('GetFilesW START');
  {$ENDIF}
  Result := 0;
  try
    {$IFDEF FAR3_UP}
    if Info.StructSize >= Sizeof(Info) then begin
    {$ELSE}
    begin
    {$ENDIF}
      if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then
        Exit;
      {$IFDEF DEBUG_LOGS}
      Log('Panel is Named Folders Panel');
      {$ENDIF}
      Item := {$IFDEF FAR3_UP} Info.PanelItem {$ELSE} PanelItem {$ENDIF} ;
      OverwriteState := osDefault;
      if {$IFDEF FAR3_UP} Info.ItemsNumber {$ELSE} ItemsNumber {$ENDIF} > 0 then begin
        for i := 0 to Pred( {$IFDEF FAR3_UP} Info.ItemsNumber {$ELSE} ItemsNumber {$ENDIF} ) do begin
          {$IFDEF DEBUG_LOGS}
          {$IFDEF FAR3_UP}
          Log('GetFilesW: Index=%d, DestPath="%s", FileName="%s", OpMode=$%x', [i, TFarUtils.CopyString(Info.DestPath), TFarUtils.CopyString(Item.FileName), Info.OpMode]);
          {$ELSE}
          Log('GetFilesW: Index=%d, DestPath="%s", FileName="%s", OpMode=$%x', [i, TFarUtils.CopyString(DestPath), TFarUtils.CopyString(Item.FindData.cFileName), OpMode]);
          {$ENDIF}
          {$ENDIF}
          if ( {$IFDEF FAR3_UP} Info.OpMode {$ELSE} OpMode {$ENDIF} and OPM_EDIT) <> 0 then begin
            Panel.EditNamedFolder(TFarUtils.CopyString( {$IFDEF FAR3_UP} Item.FileName {$ELSE} Item.FindData.cFileName {$ENDIF} ));
          end
          else begin
            if Panel.ExportNamedFolderToDirectory(
                 TFarUtils.CopyString( {$IFDEF FAR3_UP} Item.FileName {$ELSE} Item.FindData.cFileName {$ENDIF} ),
                 TFarUtils.CopyString( {$IFDEF FAR3_UP} Info.DestPath {$ELSE} DestPath {$ENDIF} ),
                 {$IFDEF FAR3_UP} Info.Move {$ELSE} Move <> 0 {$ENDIF},
                 ({$IFDEF FAR3_UP} Info.OpMode {$ELSE} OpMode {$ENDIF} and OPM_SILENT) <> 0,
                 OverwriteState
                 )
            then begin
              Item.Flags := Item.Flags and (not PPIF_SELECTED);
              Result := 1;
            end
            else begin
              Result := 0;
            end;
          end;
          Inc(Item);
        end;
      end;
    end;
    {$IFDEF DEBUG_LOGS}
    Log('Result = %d', [Result]);
    {$ENDIF}
  except
    on E: EAbort do
      Result := -1;
    on E: Exception do
      HandleException(E, 'GetFilesW');
  end;
  {$IFDEF DEBUG_LOGS}
  Log('GetFilesW END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
function PutFilesW(var Info: TPutFilesInfo): TIntPtr;
{$ELSE}
function PutFilesW(Plugin: THandle; PanelItem: PPluginPanelItem; ItemsNumber: integer; Move: integer; SrcPath: PFarChar; OpMode: integer): TIntPtr;
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
  Item: PPluginPanelItem;
  OverwriteState: TNamedFoldersPanel.TOverwriteState;
  i: Integer;
begin
  {$IFDEF DEBUG_LOGS}
  Log('PutFilesW START');
  {$ENDIF}
  Result := 0;
  try
    {$IFDEF FAR3_UP}
    if Info.StructSize >= Sizeof(Info) then begin
    {$ELSE}
    begin
    {$ENDIF}
      if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then
        Exit;
      {$IFDEF DEBUG_LOGS}
      Log('Panel is Named Folders Panel');
      {$ENDIF}
      Item := {$IFDEF FAR3_UP} Info.PanelItem {$ELSE} PanelItem {$ENDIF} ;
      OverwriteState := osDefault;
      if {$IFDEF FAR3_UP} Info.ItemsNumber {$ELSE} ItemsNumber {$ENDIF} > 0 then begin
        for i := 0 to Pred( {$IFDEF FAR3_UP} Info.ItemsNumber {$ELSE} ItemsNumber {$ENDIF} ) do begin
          {$IFDEF DEBUG_LOGS}
          {$IFDEF FAR3_UP}
          Log('PutFilesW: Index=%d, SrcPath="%s", FileName="%s", OpMode=$%x', [i, TFarUtils.CopyString(Info.SrcPath), TFarUtils.CopyString(Item.FileName), Info.OpMode]);
          {$ELSE}
          Log('PutFilesW: Index=%d, SrcPath="%s", FileName="%s", OpMode=$%x', [i, TFarUtils.CopyString(SrcPath), TFarUtils.CopyString(Item.FindData.cFileName), OpMode]);
          {$ENDIF}
          {$ENDIF}
          if Panel.ImportNamedFolderFile(
               TFarUtils.CopyString( {$IFDEF FAR3_UP} Item.FileName {$ELSE} Item.FindData.cFileName {$ENDIF} ),
               TFarUtils.CopyString( {$IFDEF FAR3_UP} Info.SrcPath {$ELSE} SrcPath {$ENDIF} ),
               Panel.Directory,
               {$IFDEF FAR3_UP} Info.Move {$ELSE} Move <> 0 {$ENDIF},
               ({$IFDEF FAR3_UP} Info.OpMode {$ELSE} OpMode {$ENDIF} and OPM_SILENT) <> 0,
               OverwriteState
               )
          then begin
            Item.Flags := Item.Flags and (not PPIF_SELECTED);
            Result := 2;
          end
          else begin
            Result := 0;
          end;
          Inc(Item);
        end;
      end;
    end;
    {$IFDEF DEBUG_LOGS}
    Log('Result = %d', [Result]);
    {$ENDIF}
  except
    on E: EAbort do
      Result := -1;
    on E: Exception do
      HandleException(E, 'PutFilesW');
  end;
  {$IFDEF DEBUG_LOGS}
  Log('PutFilesW END');
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
function ProcessPanelInputW(const Info: TProcessPanelInputInfo): TIntPtr;
{$ELSE}
function ProcessKeyW(Plugin: THandle; Key: integer; ControlState: DWORD): integer;
{$ENDIF}
const
  KeyModifiers_CTRL  = {$IFDEF FAR3_UP} LEFT_CTRL_PRESSED or RIGHT_CTRL_PRESSED {$ELSE} PKF_CONTROL {$ENDIF} ;
  KeyModifiers_ALT   = {$IFDEF FAR3_UP} LEFT_ALT_PRESSED  or RIGHT_ALT_PRESSED  {$ELSE} PKF_ALT     {$ENDIF} ;
  KeyModifiers_SHIFT = {$IFDEF FAR3_UP} SHIFT_PRESSED                           {$ELSE} PKF_SHIFT   {$ENDIF} ;
  KeyModifiers = KeyModifiers_CTRL or KeyModifiers_ALT or KeyModifiers_SHIFT;
var
  Panel: TNamedFoldersPanel;
  Handled: boolean;
  {$IFDEF FAR3_UP}
  Key, ControlState: integer;
  {$ENDIF}
begin
  Result := 0;
  try
    {$IFDEF FAR3_UP}
    if Info.StructSize >= Sizeof(Info) then begin
    {$ELSE}
    begin
    {$ENDIF}
      if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then
        Exit;
      //Log('ProcessKeyW(%x [%x], %x)', [Key, Key and (not PKF_PREPROCESS), ControlState]);
      {$IFDEF FAR3_UP}
      if Info.Rec.EventType = KEY_EVENT then begin
        Key := Info.Rec.Event.KeyEvent.wVirtualKeyCode;
        ControlState := Info.Rec.Event.KeyEvent.dwControlKeyState;
      {$ELSE}
      begin
      {$ENDIF}
        if (Key = VK_RETURN) and ((ControlState and KeyModifiers) = 0) then begin
          Result := 1;
          if Panel.ExecuteNamedFolder(True, Handled) then
            if not Handled then
              Result := 0;
        end
        {$IFDEF FAR3_UP}
          // FAR2 does not pass ALT+ENTER to the application
        else if (Key = VK_RETURN) and ((ControlState and KeyModifiers_ALT) <> 0) and ((ControlState and (KeyModifiers and (not KeyModifiers_ALT))) = 0) then begin
          Result := 1;
          Panel.ExecuteNamedFolder(False, Handled);
        end
        {$ENDIF}
        else if (Key = VK_F4) and ((ControlState and KeyModifiers) = KeyModifiers_SHIFT) then begin
          Result := 1;
          Panel.NewNamedFolder;
        end;
      end;
    end;
  except
    on E: Exception do
      HandleException(E, {$IFDEF FAR3_UP} 'ProcessPanelInputW' {$ELSE} 'ProcessKeyW' {$ENDIF} );
  end;
end;

{$IFDEF FAR3_UP}
function ProcessPanelEventW(const Info: TProcessPanelEventInfo): TIntPtr; stdcall;
{$ELSE}
function ProcessEventW(Plugin: THandle; Event: integer; Param: Pointer): TIntPtr; stdcall;
{$ENDIF}
var
  Panel: TNamedFoldersPanel;
  FileName: string;
begin
  Result := 0;
  {$IFDEF FAR3_UP}
  if Info.StructSize >= Sizeof(Info) then begin
  {$ENDIF}
    if not TNamedFoldersPanel.HandleToPanel( {$IFDEF FAR3_UP} Info.hPanel {$ELSE} Plugin {$ENDIF} , Panel) then
      Exit;
    if {$IFDEF FAR3_UP} Info. {$ENDIF} Event = FE_REDRAW then begin
      if Panel.FileToBeSelected <> '' then begin
        FileName := Panel.FileToBeSelected;
        Panel.FileToBeSelected := '';
        Panel.SelectFile(FileName);
      end;
    end;
  {$IFDEF FAR3_UP}
  end;
  {$ENDIF}
end;

function ConfigureW( {$IFDEF FAR3_UP} var Info: TConfigureInfo {$ELSE} ItemNumber: integer {$ENDIF} ): TIntPtr;
begin
  Result := 0;
  {$IFDEF FAR3_UP}
  if Info.StructSize >= Sizeof(Info) then begin
  {$ENDIF}
    //if Info.GUID^ = MENU_CONFIG_GUID then begin
      if TOptionsDialog.ExecuteDialog then
        Result := 1;
    //end;
  {$IFDEF FAR3_UP}
  end;
  {$ENDIF}
end;


exports
  SetStartupInfoW,
  ExitFARW,
  {$IFDEF FAR3_UP}
  GetGlobalInfoW,
  OpenW,
  GetOpenPanelInfoW,
  ClosePanelW,
  ProcessPanelInputW,
  ProcessPanelEventW,
  {$ELSE}
  OpenPluginW,
  GetOpenPluginInfoW,
  ClosePluginW,
  ProcessKeyW,
  ProcessEventW,
  {$ENDIF}
  GetFindDataW,
  FreeFindDataInfo,
  MakeDirectoryW,
  SetDirectoryW,
  DeleteFilesW,
  GetFilesW,
  PutFilesW,
  ConfigureW,
  GetPluginInfoW;

initialization
  {$IFDEF FAR3_UP}
  uFar.PluginID := PLUGIN_GUID;
  {$ENDIF}

end.
